@@ -322,7 +322,7 @@ find_parser (char *search_name)
 		  if ((first_nonoption_arg != NULL)
 		      && options.warnings )
 		    {
-		      /* option which folows a non-option */
+		      /* option which follows a non-option */
 		      error (0, 0,
 			     _("warning: you have specified the %s "
 			       "option after a non-option argument %s, "
@@ -756,9 +756,11 @@ tests (N can be +N or -N or N): -amin N -anewer FILE -atime N -cmin N\n\
       -wholename PATTERN -size N[bcwkMG] -true -type [bcdpflsD] -uid N\n\
       -used N -user NAME -xtype [bcdpfls]\n"));
   puts (_("\
-actions: -exec COMMAND ; -fprint FILE -fprint0 FILE -fprintf FILE FORMAT\n\
-      -fls FILE -ok COMMAND ; -print -print0 -printf FORMAT -prune -ls -delete\n\
-      -quit\n"));
+actions: -delete -print0 -printf FORMAT -fprintf FILE FORMAT -print \n\
+      -fprint0 FILE -fprint FILE -ls -fls FILE -prune -quit\n\
+      -exec COMMAND ; -exec COMMAND {} + -ok COMMAND ;\n\
+      -execdir COMMAND ; -execdir COMMAND {} + -okdir COMMAND ;\n\
+"));
   puts (_("Report (and track progress on fixing) bugs via the findutils bug-reporting\n\
 page at http://savannah.gnu.org/ or, if you have no web access, by sending\n\
 email to <bug-findutils@gnu.org>."));
@@ -2347,8 +2349,8 @@ insert_time (char **argv, int *arg_ptr, PRED_FUNC pred)
   return (true);
 }
 
-/* Get a number with comparision information.
-   The sense of the comparision information is 'normal'; that is,
+/* Get a number with comparison information.
+   The sense of the comparison information is 'normal'; that is,
    '+' looks for a count > than the number and '-' less than.
    
    STR is the ASCII representation of the number.
