@@ -13,7 +13,7 @@
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
-   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
+   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301,
    USA.
 */
 
@@ -128,7 +128,7 @@ get_new_pred_chk_op (void)
    operator. */
 
 struct predicate *
-insert_primary (boolean (*pred_func) (/* ??? */))
+insert_primary (PRED_FUNC pred_func)
 {
   struct predicate *new_pred;
 
